package net.wanji.web.service.scheme.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.dao.entity.BaseCrossSchemePO;
import net.wanji.databus.dao.entity.CrossPhaseLightsPO;
import net.wanji.databus.dao.entity.CrossPhasePO;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.dto.CrossAllSchemesDTO;
import net.wanji.databus.po.BaseCrossLightsPO;
import net.wanji.databus.po.CrossLaneLightsPO;
import net.wanji.databus.po.LaneInfoPO;
import net.wanji.web.common.constant.ControlModeConst;
import net.wanji.web.common.exception.PhaseNoNotUniqueException;
import net.wanji.web.dto.LaneIdDTO;
import net.wanji.web.dto.LightIdDTO;
import net.wanji.web.dto.SaveSchemeConfigDTO;
import net.wanji.web.mapper.scheme.CrossAccompanyPhaseMapper;
import net.wanji.web.po.scheme.CrossAccompanyPhasePO;
import net.wanji.web.service.scheme.SchemeConfigService;
import net.wanji.web.vo.scheme.LaneIdsVO;
import net.wanji.web.vo.scheme.LightIdVO;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/1/5 10:27
 */
@Service
public class SchemeConfigServiceImpl implements SchemeConfigService {
    private final BaseCrossSchemeMapper baseCrossSchemeMapper;
    private final BaseCrossPhaseMapper crossPhaseMapper;
    private final LaneInfoMapper laneInfoMapper;
    private final BaseCrossLaneLightsMapper baseCrossLaneLightsMapper;
    private final BaseCrossPhaseLightsMapper baseCrossPhaseLightsMapper;
    private final BaseCrossLightsMapper baseCrossLightsMapper;
    private final CrossAccompanyPhaseMapper crossAccompanyPhaseMapper;
    private final BaseCrossSectionMapper baseCrossSectionMapper;

    public SchemeConfigServiceImpl(BaseCrossSchemeMapper baseCrossSchemeMapper, BaseCrossPhaseMapper baseCrossPhaseMapper,
                                   LaneInfoMapper laneInfoMapper, BaseCrossLaneLightsMapper baseCrossLaneLightsMapper,
                                   BaseCrossPhaseLightsMapper baseCrossPhaseLightsMapper, BaseCrossLightsMapper baseCrossLightsMapper, CrossAccompanyPhaseMapper crossAccompanyPhaseMapper, @Qualifier("baseCrossSectionMapper") BaseCrossSectionMapper baseCrossSectionMapper) {
        this.baseCrossSchemeMapper = baseCrossSchemeMapper;
        this.crossPhaseMapper = baseCrossPhaseMapper;
        this.laneInfoMapper = laneInfoMapper;
        this.baseCrossLaneLightsMapper = baseCrossLaneLightsMapper;
        this.baseCrossPhaseLightsMapper = baseCrossPhaseLightsMapper;
        this.baseCrossLightsMapper = baseCrossLightsMapper;
        this.crossAccompanyPhaseMapper = crossAccompanyPhaseMapper;
        this.baseCrossSectionMapper = baseCrossSectionMapper;
    }

    @Override
    @Transactional
    public void saveSchemeConfig(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        Integer timeType = saveSchemeConfigDTO.getTimeType();
        if (timeType == 2) { //环式
            handleRingTime(saveSchemeConfigDTO);
        } else if (timeType == 1) { //阶段式
            // TODO: 2023/1/6 处理阶段式
        }
    }

    @Override
    public LightIdVO getLightByLane(LaneIdDTO laneIdDTO) {
        String crossId = laneIdDTO.getCrossId();
        String laneId = laneIdDTO.getLaneId();
        Integer lightId = baseCrossLaneLightsMapper.selectLightsIdByLaneId(crossId, laneId);
        LightIdVO lightIdVO = new LightIdVO();
        lightIdVO.setCrossId(crossId);
        lightIdVO.setLightId(lightId);
        return lightIdVO;
    }

    @Override
    public LaneIdsVO getLanesByLight(LightIdDTO lightIdDTO) {
        Integer lightId = lightIdDTO.getLightId();
        String crossId = lightIdDTO.getCrossId();
        LaneIdsVO laneIdsVO = new LaneIdsVO();
        laneIdsVO.setCrossId(crossId);
        laneIdsVO.setLanes(new ArrayList<>());

        // 获取方向
        BaseCrossLightsPO crossLightsPO = baseCrossLightsMapper.selectById(lightId);
        Integer dir = crossLightsPO.getDir();
        laneIdsVO.setDir(dir);
        // 行人灯类型
        Integer type = crossLightsPO.getType();
        laneIdsVO.setPedType(type);

        // 构建车道信息
        List<CrossLaneLightsPO> crossLaneLightsPOList = baseCrossLaneLightsMapper.selectLanesByLightId(crossId, lightId);
        List<String> laneIds = crossLaneLightsPOList.stream()
                .map(CrossLaneLightsPO::getLaneId)
                .collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(laneIds)) {
            List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectByLaneIds(laneIds);
            laneIdsVO.setLanes(laneInfoPOList);
        }

        return laneIdsVO;
    }

    @Override
    @Transactional
    public SaveSchemeConfigDTO listSchemeConfig(CrossIdBO crossIdBO) {
        String crossId = crossIdBO.getCrossId();
        SaveSchemeConfigDTO saveSchemeConfigDTO = new SaveSchemeConfigDTO();
        saveSchemeConfigDTO.setCrossId(crossId);
        // 构造方案列表phaseSchemeList = {ArrayList@21896}  size = 24
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = buildPhaseSchemeList(crossId);
        List<SaveSchemeConfigDTO.PhaseScheme> sort = phaseSchemeList.stream()
                .sorted(Comparator.comparing(scheme -> Integer.parseInt(scheme.getSchemeNo())))
                .collect(Collectors.toList());
        saveSchemeConfigDTO.setPhaseSchemeList(sort);
        // todo 暂时只有环式
        saveSchemeConfigDTO.setTimeType(2);
        saveSchemeConfigDTO.setCurrentSchemeId(calcCurrentSchemeId(crossId));
        return saveSchemeConfigDTO;
    }

    /**
     * 获取当前方案ID
     */
    private Integer calcCurrentSchemeId(String crossId) {
        List<CrossAllSchemesDTO> dtoList = baseCrossSectionMapper.selectCrossAllSchemes(crossId);
        // 先依据特殊日期
        // 获取今天的年月日
        Calendar todayCalendar = Calendar.getInstance();
        todayCalendar.set(Calendar.HOUR_OF_DAY, 0);
        todayCalendar.set(Calendar.MINUTE, 0);
        todayCalendar.set(Calendar.SECOND, 0);
        todayCalendar.set(Calendar.MILLISECOND, 0);
        Date today = todayCalendar.getTime();

        List<CrossAllSchemesDTO> filteredList1 = dtoList.stream()
                .filter(dto -> dto.getWeek() != null && dto.getWeek() == 0)
                .filter(dto -> {
                    if (dto.getSpecialDate() == null) {
                        return false;
                    }
                    Calendar specialDateCalendar = Calendar.getInstance();
                    specialDateCalendar.setTime(dto.getSpecialDate());
                    specialDateCalendar.set(Calendar.HOUR_OF_DAY, 0);
                    specialDateCalendar.set(Calendar.MINUTE, 0);
                    specialDateCalendar.set(Calendar.SECOND, 0);
                    specialDateCalendar.set(Calendar.MILLISECOND, 0);
                    return today.equals(specialDateCalendar.getTime()); // 今天与specialDate相同
                })
                .collect(Collectors.toList());

        if (CollectionUtil.isNotEmpty(filteredList1)) {
            return getSchemeIdFromFilteredList(filteredList1);
        }

        // 再依据星期
        // 获取今天是周几
        Calendar calendar = Calendar.getInstance();
        int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK);
        // Java Calendar 类中，周日为1，周一为2，依此类推。转换为（周一为1，周二为2... 周日为7）
        int convertedDayOfWeek = dayOfWeek == 1 ? 7 : dayOfWeek - 1;

        // 筛选出与今天星期相同的记录
        List<CrossAllSchemesDTO> filteredListWeek = dtoList.stream()
                .filter(dto -> dto.getWeek() == convertedDayOfWeek)
                .collect(Collectors.toList());

        if (CollectionUtil.isNotEmpty(filteredListWeek)) {
            return getSchemeIdFromFilteredList(filteredListWeek);
        }
        return null;
    }

    private static Integer getSchemeIdFromFilteredList(List<CrossAllSchemesDTO> filteredListWeek) {
        // 获取当前系统时间的小时和分钟
        LocalTime now = LocalTime.now();
        Optional<CrossAllSchemesDTO> result = filteredListWeek.stream()
                .filter(dto -> {
                    LocalTime start = LocalTime.parse(dto.getStartTime()); // 将 startTime 转换为 LocalTime
                    String endTime = dto.getEndTime();
                    if ("24:00".equals(endTime)) {
                        endTime = "23:59";
                    }
                    LocalTime end = LocalTime.parse(endTime); // 将 endTime 转换为 LocalTime
                    return (now.isAfter(start) || now.equals(start)) && now.isBefore(end); // 判断当前时间是否在 startTime 和 endTime 之间
                })
                .findFirst();  // 只会有一个元素
        CrossAllSchemesDTO targetDto = result.get();
        return targetDto.getSchemeId();
    }

    /**
     * 构造方案列表
     *
     * @param crossId
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.PhaseScheme>
     * @author Kent HAN
     * @date 2023/1/12 9:23
     */
    private List<SaveSchemeConfigDTO.PhaseScheme> buildPhaseSchemeList(String crossId) {
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = new ArrayList<>();
        List<BaseCrossSchemePO> baseCrossSchemePOList = baseCrossSchemeMapper.selectByCrossId(crossId);
        CrossPhasePO crossPhasePO = new CrossPhasePO();
        crossPhasePO.setCrossId(crossId);
        List<CrossPhasePO> phasePOList = crossPhaseMapper.listCrossPhasePO(crossPhasePO);
        CrossPhaseLightsPO entity = new CrossPhaseLightsPO();
        entity.setCrossId(crossId);
        List<CrossPhaseLightsPO> crossPhaseLightsList = baseCrossPhaseLightsMapper.listCrossPhaseLightsPO(entity);
        List<BaseCrossLightsPO> baseCrossLightsPOList = baseCrossLightsMapper.selectByCrossId(crossId);
        List<CrossLaneLightsPO> crossLaneLightsPOList = baseCrossLaneLightsMapper.listLightLane().stream().filter(x -> Objects.equals(x.getCrossId(), crossId)).collect(Collectors.toList());
        List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectBycrossIdAll(crossId);
        for (BaseCrossSchemePO baseCrossSchemePO : baseCrossSchemePOList) {
            SaveSchemeConfigDTO.PhaseScheme phaseScheme = new SaveSchemeConfigDTO.PhaseScheme();
            phaseScheme.setSchemeNo(baseCrossSchemePO.getSchemeNo());
            phaseScheme.setName(baseCrossSchemePO.getName());
            phaseScheme.setSchemeId(baseCrossSchemePO.getId());
            // 获取协调相位号
            Integer coordPhaseId = baseCrossSchemePO.getCoordPhase();
            if (coordPhaseId != null && CollectionUtils.isNotEmpty(phasePOList)) {
                List<CrossPhasePO> crossPhasePOList = phasePOList.stream().filter(x -> x.getId() == coordPhaseId).collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(crossPhasePOList)) {
                    CrossPhasePO phasePO = crossPhasePOList.get(0);
                    phaseScheme.setCoordPhaseNo(phasePO.getPhaseNo());
                }
            }
            phaseScheme.setCycle(baseCrossSchemePO.getCycle());
            phaseScheme.setOffset(baseCrossSchemePO.getOffset());
            phaseScheme.setStatus(baseCrossSchemePO.getStatus());
            // 构造环列表
            String schemeNo = phaseScheme.getSchemeNo();

            Integer schemeId = null;
            List<BaseCrossSchemePO> schemePOS = baseCrossSchemePOList.stream().filter(x -> x.getSchemeNo().equals(schemeNo)).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(schemePOS)) {
                BaseCrossSchemePO schemePO = schemePOS.get(0);
                schemeId = schemePO.getId();
            }
            //TODO
            List<SaveSchemeConfigDTO.RingListElement> ringList = buildRingList(crossId, schemeId);
            phaseScheme.setRingList(ringList);
            // 构造特殊控制列表-(优化后，优化前，调用的是 buildSpecialControlList)
            List<SaveSchemeConfigDTO.PhaseListElement> specialControlList = buildSpecialControlListNew(crossLaneLightsPOList, laneInfoPOList, phasePOList, crossPhaseLightsList, baseCrossLightsPOList, crossId, schemeId);
            phaseScheme.setSpecialControlModeList(specialControlList);

            phaseSchemeList.add(phaseScheme);
        }
        return phaseSchemeList;
    }

    /**
     * 通过方案号和路口号构造方案列表
     *
     * @param crossId
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.PhaseScheme>
     * @author Kent HAN
     * @date 2023/1/12 9:23
     */
    public List<SaveSchemeConfigDTO.PhaseScheme> buildPhaseSchemeListByCrossIdAndSchemeNo(String crossId, String schemeNo) {
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = new ArrayList<>();
        List<BaseCrossSchemePO> baseCrossSchemePOList = baseCrossSchemeMapper.listCrossSchemeInfo(crossId, schemeNo, null);
        for (BaseCrossSchemePO baseCrossSchemePO : baseCrossSchemePOList) {
            SaveSchemeConfigDTO.PhaseScheme phaseScheme = new SaveSchemeConfigDTO.PhaseScheme();
            phaseScheme.setSchemeNo(baseCrossSchemePO.getSchemeNo());
            phaseScheme.setName(baseCrossSchemePO.getName());
            phaseScheme.setSchemeId(baseCrossSchemePO.getId());
            // 获取协调相位号
            Integer coordPhaseId = baseCrossSchemePO.getCoordPhase();
            if (coordPhaseId != null) {
                CrossPhasePO crossPhasePO = crossPhaseMapper.selectById(coordPhaseId);
                if (crossPhasePO != null) {
                    phaseScheme.setCoordPhaseNo(crossPhasePO.getPhaseNo());
                }
            }
            phaseScheme.setCycle(baseCrossSchemePO.getCycle());
            phaseScheme.setOffset(baseCrossSchemePO.getOffset());
            phaseScheme.setStatus(baseCrossSchemePO.getStatus());
            // 构造环列表
            Integer schemeId = baseCrossSchemeMapper.selectIdByCrossIdAndSchemeNo(crossId, schemeNo);
            List<SaveSchemeConfigDTO.RingListElement> ringList = buildRingList(crossId, schemeId);
            phaseScheme.setRingList(ringList);
            // 构造特殊控制列表
            List<SaveSchemeConfigDTO.PhaseListElement> specialControlList = buildSpecialControlList(crossId, schemeId);
            phaseScheme.setSpecialControlModeList(specialControlList);

            phaseSchemeList.add(phaseScheme);
        }
        return phaseSchemeList;
    }

    /**
     * 构造特殊控制列表
     *
     * @param crossId
     * @param schemeId
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.PhaseListElement>
     * @author Kent HAN
     * @date 2023/1/12 17:53
     */
    private List<SaveSchemeConfigDTO.PhaseListElement> buildSpecialControlListNew(List<CrossLaneLightsPO> crossLaneLightsPOList, List<LaneInfoPO> laneInfoPOList, List<CrossPhasePO> phasePOList, List<CrossPhaseLightsPO> crossPhaseLightsList, List<BaseCrossLightsPO> baseCrossLightsPOList, String crossId, Integer schemeId) {
        List<SaveSchemeConfigDTO.PhaseListElement> phaseListElementList = new ArrayList<>();
        List<CrossPhasePO> crossPhasePOList = phasePOList.stream().filter(x -> x.getPlanId() == schemeId).collect(Collectors.toList());
//        List<CrossPhasePO> crossPhasePOList = crossPhaseMapper.selectByCrossIdAndSchemeId(crossId, schemeId);
        for (CrossPhasePO crossPhasePO : crossPhasePOList) {
            SaveSchemeConfigDTO.PhaseListElement phaseListElement = new SaveSchemeConfigDTO.PhaseListElement();
            Integer controlMode = crossPhasePO.getControlMode();
            if (Objects.equals(controlMode, ControlModeConst.IS_GREEN)
                    || Objects.equals(controlMode, ControlModeConst.IS_RED)
                    || Objects.equals(controlMode, ControlModeConst.GREEN_FLASH)
                    || Objects.equals(controlMode, ControlModeConst.YELLOW_FLASH)) {
                String phaseNo = crossPhasePO.getPhaseNo();
                phaseListElement.setPhaseNo(phaseNo);
                phaseListElement.setPhaseType(crossPhasePO.getPhaseType());
                phaseListElement.setMinGreenTime(crossPhasePO.getMinGreenTime());
                phaseListElement.setMaxGreenTime(crossPhasePO.getMaxGreenTime());
                phaseListElement.setGreenTime(crossPhasePO.getGreenTime());
                phaseListElement.setGreenFlashTime(crossPhasePO.getGreenFlashTime());
                phaseListElement.setYellowFlashTime(crossPhasePO.getYellowFlashTime());
                phaseListElement.setRedFlashTime(crossPhasePO.getRedFlashTime());
                phaseListElement.setYellowTime(crossPhasePO.getYellowTime());
                phaseListElement.setRedTime(crossPhasePO.getRedTime());
                phaseListElement.setYellowFlash(crossPhasePO.getYellowFlashTime());
                if (controlMode.equals(ControlModeConst.IS_RED)) {
                    phaseListElement.setIsRed(1);
                } else if (controlMode.equals(ControlModeConst.IS_GREEN)) {
                    phaseListElement.setIsGreen(1);
                } else if (controlMode.equals(ControlModeConst.GREEN_FLASH)) {
                    phaseListElement.setGreenFlash(1);
                } else if (controlMode.equals(ControlModeConst.YELLOW_FLASH)) {
                    phaseListElement.setYellowFlash(1);
                }
                phaseListElement.setAccompanyPhaseNoList(new ArrayList<>());
//                Integer phaseId = crossPhaseMapper.selectIdByCrossIdAndPhaseNo(crossId, schemeId, phaseNo);
                Integer phaseId = null;
                List<CrossPhasePO> phasePOS = phasePOList.stream().filter(x -> x.getPlanId() == schemeId && phaseNo.equals(x.getPhaseNo())).collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(phasePOS)) {
                    CrossPhasePO crossPhasePO1 = phasePOS.get(0);
                    phaseId = crossPhasePO1.getId();
                }
//                List<SaveSchemeConfigDTO.DirListElement> dirList = buildDirList(crossId, phaseId);
                List<SaveSchemeConfigDTO.DirListElement> dirList = buildDirListNew(crossLaneLightsPOList, laneInfoPOList, crossPhaseLightsList, baseCrossLightsPOList, crossId, phaseId);
                phaseListElement.setDirList(dirList);

                phaseListElementList.add(phaseListElement);
            }
        }
        return phaseListElementList;
    }

    /**
     * 构造特殊控制列表
     *
     * @param crossId
     * @param schemeId
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.PhaseListElement>
     * @author Kent HAN
     * @date 2023/1/12 17:53
     */
    private List<SaveSchemeConfigDTO.PhaseListElement> buildSpecialControlList(String crossId, Integer schemeId) {
        List<SaveSchemeConfigDTO.PhaseListElement> phaseListElementList = new ArrayList<>();

        List<CrossPhasePO> crossPhasePOList = crossPhaseMapper.selectByCrossIdAndSchemeId(crossId, schemeId);
        CrossPhaseLightsPO entity = new CrossPhaseLightsPO();
        entity.setCrossId(crossId);
        List<CrossPhaseLightsPO> crossPhaseLightsList = baseCrossPhaseLightsMapper.listCrossPhaseLightsPO(entity);
        List<BaseCrossLightsPO> baseCrossLightsPOS = baseCrossLightsMapper.selectByCrossId(crossId);
        List<CrossLaneLightsPO> crossLaneLightsPOList = baseCrossLaneLightsMapper.selectByCrossId(crossId);
        List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectBycrossIdAll(crossId);
        for (CrossPhasePO crossPhasePO : crossPhasePOList) {
            SaveSchemeConfigDTO.PhaseListElement phaseListElement = new SaveSchemeConfigDTO.PhaseListElement();
            Integer controlMode = crossPhasePO.getControlMode();
            if (Objects.equals(controlMode, ControlModeConst.IS_GREEN)
                    || Objects.equals(controlMode, ControlModeConst.IS_RED)
                    || Objects.equals(controlMode, ControlModeConst.GREEN_FLASH)
                    || Objects.equals(controlMode, ControlModeConst.YELLOW_FLASH)) {
                String phaseNo = crossPhasePO.getPhaseNo();
                phaseListElement.setPhaseNo(phaseNo);
                phaseListElement.setPhaseType(crossPhasePO.getPhaseType());
                phaseListElement.setMinGreenTime(crossPhasePO.getMinGreenTime());
                phaseListElement.setMaxGreenTime(crossPhasePO.getMaxGreenTime());
                phaseListElement.setGreenTime(crossPhasePO.getGreenTime());
                phaseListElement.setGreenFlashTime(crossPhasePO.getGreenFlashTime());
                phaseListElement.setYellowFlashTime(crossPhasePO.getYellowFlashTime());
                phaseListElement.setRedFlashTime(crossPhasePO.getRedFlashTime());
                phaseListElement.setYellowTime(crossPhasePO.getYellowTime());
                phaseListElement.setRedTime(crossPhasePO.getRedTime());
                phaseListElement.setYellowFlash(crossPhasePO.getYellowFlashTime());
                if (controlMode.equals(ControlModeConst.IS_RED)) {
                    phaseListElement.setIsRed(1);
                } else if (controlMode.equals(ControlModeConst.IS_GREEN)) {
                    phaseListElement.setIsGreen(1);
                } else if (controlMode.equals(ControlModeConst.GREEN_FLASH)) {
                    phaseListElement.setGreenFlash(1);
                } else if (controlMode.equals(ControlModeConst.YELLOW_FLASH)) {
                    phaseListElement.setYellowFlash(1);
                }
                phaseListElement.setAccompanyPhaseNoList(new ArrayList<>());
                Integer phaseId = crossPhaseMapper.selectIdByCrossIdAndPhaseNo(crossId, schemeId, phaseNo);
                List<SaveSchemeConfigDTO.DirListElement> dirList = buildDirList(crossPhaseLightsList,baseCrossLightsPOS,crossLaneLightsPOList,laneInfoPOList,crossId, phaseId);
                phaseListElement.setDirList(dirList);

                phaseListElementList.add(phaseListElement);
            }
        }
        return phaseListElementList;
    }

    /**
     * 构造环列表
     *
     * @param crossId
     * @param schemeId
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.RingListElement>
     * @author Kent HAN
     * @date 2023/1/12 9:28
     */
    private List<SaveSchemeConfigDTO.RingListElement> buildRingList(String crossId, Integer schemeId) {
        List<SaveSchemeConfigDTO.RingListElement> ringlist = new ArrayList<>();
        List<Integer> ringNoList = crossPhaseMapper.selectRingNumbersByCrossIdAndSchemeId(crossId, schemeId);
        for (Integer ringNo : ringNoList) {
            SaveSchemeConfigDTO.RingListElement ringListElement = new SaveSchemeConfigDTO.RingListElement();
            if (ringNo != 0) {
                ringListElement.setRingNo(ringNo);
                // 构造组列表
                List<SaveSchemeConfigDTO.GroupListElement> groupList = buildGroupList(crossId, schemeId, ringNo);
                ringListElement.setGroupList(groupList);
                ringlist.add(ringListElement);
            }
        }
        return ringlist;
    }

    /**
     * 构造组列表
     *
     * @param crossId
     * @param schemeId
     * @param ringNo
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.GroupListElement>
     * @author Kent HAN
     * @date 2023/1/12 9:42
     */
    private List<SaveSchemeConfigDTO.GroupListElement> buildGroupList(
            String crossId, Integer schemeId, Integer ringNo) {
        List<SaveSchemeConfigDTO.GroupListElement> groupList = new ArrayList<>();
        List<Integer> groupNumbers = crossPhaseMapper.selectGroupNumbers(crossId, schemeId, ringNo);
        for (Integer groupNumber : groupNumbers) {
            SaveSchemeConfigDTO.GroupListElement groupListElement = new SaveSchemeConfigDTO.GroupListElement();
            groupListElement.setGroupNo(groupNumber);
            // 构造相位列表
            List<SaveSchemeConfigDTO.PhaseListElement> phaseList =
                    buildPhaseList(crossId, schemeId, ringNo, groupNumber);
            groupListElement.setPhaseList(phaseList);

            groupList.add(groupListElement);
        }
        return groupList;
    }

    /**
     * 构造相位列表
     *
     * @param crossId
     * @param schemeId
     * @param ringNo
     * @param groupNumber
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.PhaseListElement>
     * @author Kent HAN
     * @date 2023/1/12 14:58
     */
    private List<SaveSchemeConfigDTO.PhaseListElement> buildPhaseList(
            String crossId, Integer schemeId, Integer ringNo, Integer groupNumber) {
        List<SaveSchemeConfigDTO.PhaseListElement> phaseList = new ArrayList<>();
//        List<CrossPhasePO> crossPhasePOList = crossPhaseMapper.selectByGroupNo(crossId, schemeId, ringNo, groupNumber);
        CrossPhasePO entity = new CrossPhasePO();
        entity.setCrossId(crossId);
        List<CrossPhasePO> phasePOList = crossPhaseMapper.listCrossPhasePO(entity);
        CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
        crossPhaseLightsPO.setCrossId(crossId);
        List<CrossPhaseLightsPO> crossPhaseLightsList = baseCrossPhaseLightsMapper.listCrossPhaseLightsPO(crossPhaseLightsPO);
        List<BaseCrossLightsPO> baseCrossLightsPOS = baseCrossLightsMapper.selectByCrossId(crossId);

        List<CrossLaneLightsPO> crossLaneLightsPOList = baseCrossLaneLightsMapper.selectByCrossId(crossId);
        List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectBycrossIdAll(crossId);

        if (CollectionUtil.isNotEmpty(phasePOList)) {
            phasePOList = phasePOList.stream()
                    .sorted(Comparator.comparing(CrossPhasePO::getSort))
                    .collect(Collectors.toList());

            List<CrossPhasePO> phasePOS = phasePOList.stream()
                    .filter(x -> Objects.equals(x.getPlanId(), schemeId) && Objects.equals(x.getRingNo(), ringNo) && Objects.equals(x.getGroupNo(), groupNumber))
                    .collect(Collectors.toList());

            for (CrossPhasePO crossPhasePO : phasePOS) {
                SaveSchemeConfigDTO.PhaseListElement phaseListElement = new SaveSchemeConfigDTO.PhaseListElement();
                phaseListElement.setPhaseId(crossPhasePO.getId());
                String phaseNo = crossPhasePO.getPhaseNo();
                phaseListElement.setPhaseNo(phaseNo);
                Integer phaseType = crossPhasePO.getPhaseType();
                phaseListElement.setPhaseType(phaseType);
                phaseListElement.setMinGreenTime(crossPhasePO.getMinGreenTime());
                phaseListElement.setMaxGreenTime(crossPhasePO.getMaxGreenTime());
                phaseListElement.setGreenTime(crossPhasePO.getGreenTime());
                phaseListElement.setGreenFlashTime(crossPhasePO.getGreenFlashTime());
                phaseListElement.setYellowFlashTime(crossPhasePO.getYellowFlashTime());
                phaseListElement.setRedFlashTime(crossPhasePO.getRedFlashTime());
                phaseListElement.setYellowTime(crossPhasePO.getYellowTime());
                phaseListElement.setRedTime(crossPhasePO.getRedTime());
                // 特殊控制
                Integer controlMode = crossPhasePO.getControlMode();
                if (Objects.equals(controlMode, ControlModeConst.YELLOW_FLASH)) {
                    phaseListElement.setYellowFlash(1);
                } else if (Objects.equals(controlMode, ControlModeConst.IS_RED)) {
                    phaseListElement.setIsRed(1);
                } else if (Objects.equals(controlMode, ControlModeConst.IS_GREEN)) {
                    phaseListElement.setIsGreen(1);
                } else if (Objects.equals(controlMode, ControlModeConst.GREEN_FLASH)) {
                    phaseListElement.setGreenFlash(1);
                }

                // 伴随相位的母相位列表
//                Integer phaseId = crossPhaseMapper.selectIdByCrossIdAndPhaseNo(crossId, schemeId, phaseNo);
                List<CrossPhasePO> crossPhasePOS = phasePOS.stream()
                        .filter(x -> Objects.equals(x.getPlanId(), schemeId) && Objects.equals(x.getPhaseNo(), phaseNo))
                        .collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(crossPhasePOS)) {
                    CrossPhasePO phasePO = crossPhasePOS.get(0);
                    Integer phaseId = phasePO.getId();
                    /*
                     //暂时不考虑伴随相位
                    if (phaseType == 3) {
                        List<String> superPhaseNoList = new ArrayList<>();
                        //TODO 可以提出来
                        List<Integer> superIdList = crossAccompanyPhaseMapper.selectByPhaseId(phaseId);
                        //TODO 可以提出来
                        List<CrossPhasePO> superList = crossPhaseMapper.selectByIds(superIdList);
                        for (CrossPhasePO phasePO : superList) {
                            superPhaseNoList.add(phasePO.getPhaseNo());
                        }
                        phaseListElement.setAccompanyPhaseNoList(superPhaseNoList);
                    }*/
                    // 构造方向列表
                    List<SaveSchemeConfigDTO.DirListElement> dirList = buildDirList(crossPhaseLightsList,baseCrossLightsPOS,crossLaneLightsPOList,laneInfoPOList,crossId, phaseId);
                    phaseListElement.setDirList(dirList);

                    phaseList.add(phaseListElement);

                }

            }
        }

        return phaseList;
    }

    /**
     * 构造方向列表
     *
     * @param crossId
     * @param phaseId
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.DirListElement>
     * @author Kent HAN
     * @date 2023/1/12 15:02
     */
    private List<SaveSchemeConfigDTO.DirListElement> buildDirListNew(List<CrossLaneLightsPO> crossLaneLightsPOList, List<LaneInfoPO> laneInfoPOList, List<CrossPhaseLightsPO> crossPhaseLightsList, List<BaseCrossLightsPO> baseCrossLightsPOList, String crossId, Integer phaseId) {
        List<SaveSchemeConfigDTO.DirListElement> dirList = new ArrayList<>();
        /*CrossPhaseLightsPO entity = new CrossPhaseLightsPO();
        entity.setPhaseId(phaseId);
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = baseCrossPhaseLightsMapper.listCrossPhaseLightsPO(entity);
*/
        if (CollectionUtil.isNotEmpty(crossPhaseLightsList)) {
            List<CrossPhaseLightsPO> crossPhaseLightsPOList = crossPhaseLightsList.stream().filter(x -> x.getPhaseId().equals(phaseId)).collect(Collectors.toList());
            Set<Integer> dirSet = new HashSet<>();
            for (CrossPhaseLightsPO crossPhaseLightsPO : crossPhaseLightsPOList) {
                Integer lightsId = crossPhaseLightsPO.getLightsId();
                /*BaseCrossLightsPO crossLightsPO = baseCrossLightsMapper.selectById(lightsId);
                if (crossLightsPO != null) {
                    Integer dir = crossLightsPO.getDir();
                    dirSet.add(dir);
                }*/
                if (CollectionUtils.isNotEmpty(baseCrossLightsPOList)) {
                    List<BaseCrossLightsPO> collect = baseCrossLightsPOList.stream().filter(x -> x.getId().equals(lightsId)).collect(Collectors.toList());
                    if (CollectionUtils.isNotEmpty(collect)) {
                        BaseCrossLightsPO crossLightsPO = collect.get(0);
                        Integer dir = crossLightsPO.getDir();
                        dirSet.add(dir);
                    }
                }

            }
            for (Integer dir : dirSet) {
                SaveSchemeConfigDTO.DirListElement dirListElement = new SaveSchemeConfigDTO.DirListElement();
                // 前端灯组方向就是车道方向
                dirListElement.setDir(dir);
                // 是否有行人相位
//                List<CrossPhaseLightsPO> crossPhaseLightsPOList1 = baseCrossPhaseLightsMapper.selectByPhaseId(phaseId);
                List<CrossPhaseLightsPO> crossPhaseLightsPOList1 = crossPhaseLightsList.stream().filter(x -> x.getPhaseId().equals(phaseId)).collect(Collectors.toList());
                for (CrossPhaseLightsPO crossPhaseLightsPO : crossPhaseLightsPOList1) {
                    Integer lightsId = crossPhaseLightsPO.getLightsId();
//                    BaseCrossLightsPO baseCrossLightsPO = baseCrossLightsMapper.selectById(lightsId);
                    List<BaseCrossLightsPO> collect = baseCrossLightsPOList.stream().filter(x -> x.getId().equals(lightsId)).collect(Collectors.toList());
                    BaseCrossLightsPO baseCrossLightsPO = collect.get(0);
                    Integer lightsDir = baseCrossLightsPO.getDir();
                    Integer type = baseCrossLightsPO.getType();
                    if (type >= 20 && type < 30 && Objects.equals(lightsDir, dir)) {
                        dirListElement.setHasPersonPhase(1);
                    }
                }
                /*List<BaseCrossLightsPO> crossLightsPOList = baseCrossLightsMapper.selectByCrossIdAndDir(crossId, dir);
                for (BaseCrossLightsPO crossLightsPO : crossLightsPOList) {
                    Integer type = crossLightsPO.getType();
                }*/
                // 构造车道列表
//                List<SaveSchemeConfigDTO.LaneListElement> laneList = buildLaneList(dir, phaseId, crossId);
                List<SaveSchemeConfigDTO.LaneListElement> laneList = buildLaneListNew(crossLaneLightsPOList, laneInfoPOList, crossPhaseLightsList, baseCrossLightsPOList, dir, phaseId, crossId);
                dirListElement.setLaneList(laneList);

                dirList.add(dirListElement);
            }
        }


        return dirList;
    }

    /**
     * 构造方向列表
     *
     * @param crossId
     * @param phaseId
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.DirListElement>
     * @author Kent HAN
     * @date 2023/1/12 15:02
     */
    private List<SaveSchemeConfigDTO.DirListElement> buildDirList(List<CrossPhaseLightsPO> crossPhaseLightsList,List<BaseCrossLightsPO> baseCrossLightsPOS,List<CrossLaneLightsPO> crossLaneLightsPOList,List<LaneInfoPO> laneInfoPOList,String crossId, Integer phaseId) {
        List<SaveSchemeConfigDTO.DirListElement> dirList = new ArrayList<>();
        /*CrossPhaseLightsPO entity = new CrossPhaseLightsPO();
        entity.setPhaseId(phaseId);
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = baseCrossPhaseLightsMapper.listCrossPhaseLightsPO(entity);*/

        if (CollectionUtil.isNotEmpty(crossPhaseLightsList)){
            List<CrossPhaseLightsPO> crossPhaseLightsPOList = crossPhaseLightsList.stream().filter(x -> Objects.equals(x.getPhaseId(), phaseId)).collect(Collectors.toList());
            Set<Integer> dirSet = new HashSet<>();
            for (CrossPhaseLightsPO crossPhaseLightsPO : crossPhaseLightsPOList) {
                Integer lightsId = crossPhaseLightsPO.getLightsId();
                /*BaseCrossLightsPO crossLightsPO = baseCrossLightsMapper.selectById(lightsId);
                if (crossLightsPO != null) {
                    Integer dir = crossLightsPO.getDir();
                    dirSet.add(dir);
                }*/
                List<BaseCrossLightsPO> collected = baseCrossLightsPOS.stream().filter(x -> Objects.equals(x.getId(), lightsId)).collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(collected)){
                    BaseCrossLightsPO lightsPO = collected.get(0);
                    Integer dir = lightsPO.getDir();
                    dirSet.add(dir);
                }
            }
            for (Integer dir : dirSet) {
                SaveSchemeConfigDTO.DirListElement dirListElement = new SaveSchemeConfigDTO.DirListElement();
                // 前端灯组方向就是车道方向
//          Integer laneDir = CrossLisghtsLaneDirEnum.getLaneDirByLightsDir(dir);
                dirListElement.setDir(dir);
                // 是否有行人相位
//                List<CrossPhaseLightsPO> crossPhaseLightsPOList1 = baseCrossPhaseLightsMapper.selectByPhaseId(phaseId);
                //重复代码，crossPhaseLightsPOList和crossPhaseLightsPOList1 是同一个结果集
                for (CrossPhaseLightsPO crossPhaseLightsPO : crossPhaseLightsPOList) {
                    Integer lightsId = crossPhaseLightsPO.getLightsId();
                   /* BaseCrossLightsPO baseCrossLightsPO = baseCrossLightsMapper.selectById(lightsId);
                    Integer lightsDir = baseCrossLightsPO.getDir();
                    Integer type = baseCrossLightsPO.getType();
                    if (type >= 20 && type < 30 && Objects.equals(lightsDir, dir)) {
                        dirListElement.setHasPersonPhase(1);
                    }*/

                    List<BaseCrossLightsPO> collected = baseCrossLightsPOS.stream().filter(x -> Objects.equals(x.getId(), lightsId)).collect(Collectors.toList());
                    if (CollectionUtil.isNotEmpty(collected)){
                        BaseCrossLightsPO lightsPO = collected.get(0);
                        Integer lightsDir = lightsPO.getDir();
                        Integer type = lightsPO.getType();
                        if (type >= 20 && type < 30 && Objects.equals(lightsDir, dir)) {
                            dirListElement.setHasPersonPhase(1);
                        }
                        dirSet.add(dir);
                    }
                }
                //注释无效代码
           /* List<BaseCrossLightsPO> crossLightsPOList = baseCrossLightsMapper.selectByCrossIdAndDir(crossId, dir);
            for (BaseCrossLightsPO crossLightsPO : crossLightsPOList) {
                Integer type = crossLightsPO.getType();
            }*/
                // 构造车道列表
                List<SaveSchemeConfigDTO.LaneListElement> laneList = buildLaneList(crossPhaseLightsList,baseCrossLightsPOS,crossLaneLightsPOList,laneInfoPOList,dir, phaseId, crossId);
                dirListElement.setLaneList(laneList);

                dirList.add(dirListElement);
            }
        }
        return dirList;
    }

    /**
     * 构造车道列表
     *
     * @param dir
     * @param phaseId
     * @param crossId
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.LaneListElement>
     * @author Kent HAN
     * @date 2023/1/12 15:02
     */
    private List<SaveSchemeConfigDTO.LaneListElement> buildLaneListNew(List<CrossLaneLightsPO> crossLaneLightsPOList, List<LaneInfoPO> laneInfoPOList, List<CrossPhaseLightsPO> crossPhaseLightsList, List<BaseCrossLightsPO> baseCrossLightsPOList, Integer dir, Integer phaseId, String crossId) {
        List<SaveSchemeConfigDTO.LaneListElement> laneList = new ArrayList<>();
        /*CrossPhaseLightsPO entity = new CrossPhaseLightsPO();
        entity.setPhaseId(phaseId);
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = baseCrossPhaseLightsMapper.listCrossPhaseLightsPO(entity);
        List<Integer> lightsIds = crossPhaseLightsPOList.stream()
                .map(CrossPhaseLightsPO::getLightsId)
                .collect(Collectors.toList());
        List<BaseCrossLightsPO> crossLightsPOList = baseCrossLightsMapper.selectByLightsIdsAndDir(lightsIds, dir);
                */
        List<BaseCrossLightsPO> crossLightsPOList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(crossPhaseLightsList)) {
            List<Integer> lightsIds = crossPhaseLightsList.stream().filter(x -> Objects.equals(x.getPhaseId(), phaseId)).map(CrossPhaseLightsPO::getLightsId)
                    .collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(lightsIds)) {
                crossLightsPOList = baseCrossLightsPOList.stream().filter(x -> Objects.equals(x.getDir(), dir) && lightsIds.contains(x.getId())).collect(Collectors.toList());
            }
        }

        for (BaseCrossLightsPO crossLightsPO : crossLightsPOList) {
            Integer type = crossLightsPO.getType();
            if (type < 20 || type >= 30) { // 非行人灯
                Integer lightsId = crossLightsPO.getId();

                /*List<CrossLaneLightsPO> crossLaneLightsPOList = baseCrossLaneLightsMapper.selectByLightsId(lightsId);
                List<String> laneIds = crossLaneLightsPOList.stream()
                        .map(CrossLaneLightsPO::getLaneId)
                        .collect(Collectors.toList());*/

                if (CollectionUtils.isNotEmpty(crossLaneLightsPOList)) {
                    List<String> laneIds = crossLaneLightsPOList.stream()
                            .filter(x -> Objects.equals(x.getLightsId(), lightsId))
                            .map(CrossLaneLightsPO::getLaneId)
                            .collect(Collectors.toList());

                    if (CollectionUtil.isNotEmpty(laneIds) && CollectionUtil.isNotEmpty(laneInfoPOList)) {
//                        List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectByLaneIds(laneIds);
                        laneInfoPOList = laneInfoPOList.stream().filter(x -> laneIds.contains(x.getId())).collect(Collectors.toList());

                        // 根据sort排序
                        List<LaneInfoPO> collect = laneInfoPOList.stream()
                                .sorted(Comparator.comparing(LaneInfoPO::getSort))
                                .collect(Collectors.toList());
                        // 根据转向去重
                        TreeSet<LaneInfoPO> laneInfoPOTreeSet = collect.stream().collect(Collectors.toCollection(
                                () -> new TreeSet<>(Comparator.comparing(LaneInfoPO::getTurn))));
                        for (LaneInfoPO laneInfoPO : laneInfoPOTreeSet) {
                            SaveSchemeConfigDTO.LaneListElement laneListElement = new SaveSchemeConfigDTO.LaneListElement();
                            laneListElement.setDirection(laneInfoPO.getTurn());
                            laneListElement.setLaneType(laneInfoPO.getCategory());
                            laneListElement.setLaneId(laneInfoPO.getId());
                            laneListElement.setLightsId(lightsId);
                            laneList.add(laneListElement);
                        }
                    }
                }
            }
        }
        List<SaveSchemeConfigDTO.LaneListElement> sortedList = laneList.stream()
                .sorted(Comparator.comparing(SaveSchemeConfigDTO.LaneListElement::getLaneId))
                .collect(Collectors.toList());
        return sortedList;
    }

    /**
     * 构造车道列表
     *
     * @param dir
     * @param phaseId
     * @param crossId
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.LaneListElement>
     * @author Kent HAN
     * @date 2023/1/12 15:02
     */
    private List<SaveSchemeConfigDTO.LaneListElement> buildLaneList(List<CrossPhaseLightsPO> crossPhaseLightsList,List<BaseCrossLightsPO> baseCrossLightsPOSList,List<CrossLaneLightsPO> crossLaneLightsPOList,List<LaneInfoPO> laneInfoPOList,Integer dir, Integer phaseId, String crossId) {
        List<SaveSchemeConfigDTO.LaneListElement> laneList = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(crossPhaseLightsList)){
            List<CrossPhaseLightsPO> crossPhaseLightsPOList = crossPhaseLightsList.stream().filter(x -> Objects.equals(x.getPhaseId(), phaseId)).collect(Collectors.toList());

            /*CrossPhaseLightsPO entity = new CrossPhaseLightsPO();
            entity.setPhaseId(phaseId);
            List<CrossPhaseLightsPO> crossPhaseLightsPOList = baseCrossPhaseLightsMapper.listCrossPhaseLightsPO(entity);*/
            List<Integer> lightsIds = crossPhaseLightsPOList.stream()
                    .map(CrossPhaseLightsPO::getLightsId)
                    .collect(Collectors.toList());
//            List<BaseCrossLightsPO> crossLightsPOList = baseCrossLightsMapper.selectByLightsIdsAndDir(lightsIds, dir);

            if (CollectionUtil.isNotEmpty(baseCrossLightsPOSList) && CollectionUtil.isNotEmpty(lightsIds)){
                List<BaseCrossLightsPO> crossLightsPOList = baseCrossLightsPOSList.stream().filter(x -> Objects.equals(x.getDir(), dir) && lightsIds.contains(x.getId())).collect(Collectors.toList());
                for (BaseCrossLightsPO crossLightsPO : crossLightsPOList) {
                    Integer type = crossLightsPO.getType();
                    if (type < 20 || type >= 30) { // 非行人灯
                        Integer lightsId = crossLightsPO.getId();
                        /*List<CrossLaneLightsPO> crossLaneLightsPOList = baseCrossLaneLightsMapper.selectByLightsId(lightsId);
                        List<String> laneIds = crossLaneLightsPOList.stream()
                                .map(CrossLaneLightsPO::getLaneId)
                                .collect(Collectors.toList());*/

                        if (CollectionUtil.isNotEmpty(crossLaneLightsPOList)){
                            List<String> laneIds = crossLaneLightsPOList.stream()
                                    .filter(x -> Objects.equals(x.getLightsId(), lightsId))
                                    .map(CrossLaneLightsPO::getLaneId)
                                    .collect(Collectors.toList());
                            if (CollectionUtil.isNotEmpty(laneIds) && CollectionUtil.isNotEmpty(laneInfoPOList)) {
                               /* List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectByLaneIds(laneIds);
                                // 根据sort排序
                                List<LaneInfoPO> collect = laneInfoPOList.stream()
                                        .sorted(Comparator.comparing(LaneInfoPO::getSort))
                                        .collect(Collectors.toList());*/

                                List<LaneInfoPO>  laneInfoList = laneInfoPOList.stream()
                                        .filter(x -> laneIds.contains(x.getId()))
                                        .sorted(Comparator.comparing(LaneInfoPO::getSort))
                                        .collect(Collectors.toList());
                                if (CollectionUtil.isNotEmpty(laneInfoList)){
                                    // 根据转向去重
                                    TreeSet<LaneInfoPO> laneInfoPOTreeSet = laneInfoList.stream().collect(Collectors.toCollection(
                                            () -> new TreeSet<>(Comparator.comparing(LaneInfoPO::getTurn))));
                                    for (LaneInfoPO laneInfoPO : laneInfoPOTreeSet) {
                                        SaveSchemeConfigDTO.LaneListElement laneListElement = new SaveSchemeConfigDTO.LaneListElement();
                                        laneListElement.setDirection(laneInfoPO.getTurn());
                                        laneListElement.setLaneType(laneInfoPO.getCategory());
                                        laneListElement.setLaneId(laneInfoPO.getId());
                                        laneListElement.setLightsId(lightsId);
                                        laneList.add(laneListElement);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        List<SaveSchemeConfigDTO.LaneListElement> sortedList = laneList.stream()
                .sorted(Comparator.comparing(SaveSchemeConfigDTO.LaneListElement::getLaneId))
                .collect(Collectors.toList());
        return sortedList;
    }

    private boolean hasLightsId(Integer lightsId, Integer phaseId, String crossId) {
        CrossPhaseLightsPO entity = new CrossPhaseLightsPO();
        entity.setPhaseId(phaseId);
        entity.setCrossId(crossId);
        entity.setLightsId(lightsId);
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = baseCrossPhaseLightsMapper.listCrossPhaseLightsPO(entity);
        return !CollectionUtil.isEmpty(crossPhaseLightsPOList);
    }

    private void handleRingTime(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        // 更新方案基础信息表
        updateCrossScheme(saveSchemeConfigDTO);
        // 更新相位基础表、相位灯组关系表、伴随相位母相位关系
        updateCrossPhase(saveSchemeConfigDTO);
        // 更新协调相位ID
        updateCoordPhase(saveSchemeConfigDTO);
    }

    private void updateCoordPhase(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        String crossId = saveSchemeConfigDTO.getCrossId();
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = saveSchemeConfigDTO.getPhaseSchemeList();
        for (SaveSchemeConfigDTO.PhaseScheme phaseScheme : phaseSchemeList) {
            String schemeNo = phaseScheme.getSchemeNo();
            Integer schemeId = baseCrossSchemeMapper.selectIdByCrossIdAndSchemeNo(crossId, schemeNo);
            String coordPhaseNo = phaseScheme.getCoordPhaseNo();
            Integer phaseId = crossPhaseMapper.selectIdByCrossIdAndPhaseNo(crossId, schemeId, coordPhaseNo);
            if (phaseId != null) {
                baseCrossSchemeMapper.updateCoordPhase(crossId, schemeNo, phaseId);
            }
        }
    }

    private void updateCrossPhase(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        String crossId = saveSchemeConfigDTO.getCrossId();
        crossPhaseMapper.deleteByCrossId(crossId);
        baseCrossPhaseLightsMapper.deleteByCrossId(crossId);
        crossAccompanyPhaseMapper.deleteByCrossId(crossId);

        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = saveSchemeConfigDTO.getPhaseSchemeList();
        for (SaveSchemeConfigDTO.PhaseScheme phaseScheme : phaseSchemeList) {
            // 用于限制相位号不能重复
            Set<String> phaseNoSet = new HashSet<>();
            String schemeNo = phaseScheme.getSchemeNo();
            // 获取方案ID
            Integer schemeId = baseCrossSchemeMapper.selectIdByCrossIdAndSchemeNo(crossId, schemeNo);
            List<SaveSchemeConfigDTO.RingListElement> ringList = phaseScheme.getRingList();
            // 用于维护伴随相位
            Map<String, SaveSchemeConfigDTO.PhaseListElement> phaseNoMap = new HashMap<>();
            for (SaveSchemeConfigDTO.RingListElement ringListElement : ringList) {
                Integer ringNo = ringListElement.getRingNo();
                List<SaveSchemeConfigDTO.GroupListElement> groupList = ringListElement.getGroupList();
                for (int i = 0; i < groupList.size(); i++) {
                    Integer groupNo = i + 1;
                    SaveSchemeConfigDTO.GroupListElement groupListElement = groupList.get(i);
                    List<SaveSchemeConfigDTO.PhaseListElement> phaseList = groupListElement.getPhaseList();
                    for (int j = 0; j < phaseList.size(); j++) {
                        SaveSchemeConfigDTO.PhaseListElement phase = phaseList.get(j);
                        String phaseNo = phase.getPhaseNo();
                        try {
                            Integer.parseInt(phaseNo);
                        } catch (Exception e) {
                            throw new RuntimeException("相位编号必须为数字");
                        }
                        testPhaseNo(phaseNoSet, phaseNo);
                        Integer phaseType = phase.getPhaseType();
                        if (phaseType != 3) { // 实相位或虚相位
                            phaseNoMap.put(phaseNo, phase);
                            Integer yellowFlashTime = phase.getYellowFlashTime();
                            Integer redFlashTime = phase.getRedFlashTime();
                            Integer greenTime = phase.getGreenTime();
                            Integer greenFlashTime = phase.getGreenFlashTime();
                            Integer yellowTime = phase.getYellowTime();
                            Integer redTime = phase.getRedTime();
                            Integer minGreenTime = phase.getMinGreenTime();
                            Integer maxGreenTime = phase.getMaxGreenTime();

                            CrossPhasePO crossPhasePO = new CrossPhasePO();
                            crossPhasePO.setPhaseNo(phaseNo);
                            crossPhasePO.setPhaseType(phaseType);
                            crossPhasePO.setSort(j + 1);
                            crossPhasePO.setCrossId(crossId);
                            crossPhasePO.setPlanId(schemeId);
                            crossPhasePO.setRingNo(ringNo);
                            crossPhasePO.setGroupNo(groupNo);
                            // 控制模式
                            crossPhasePO.setControlMode(ControlModeConst.FIXED_CYCLE);

                            crossPhasePO.setGreenTime(greenTime);
                            crossPhasePO.setGreenFlashTime(greenFlashTime);
                            crossPhasePO.setYellowTime(yellowTime);
                            crossPhasePO.setRedTime(redTime);
                            crossPhasePO.setYellowFlashTime(yellowFlashTime);
                            crossPhasePO.setRedFlashTime(redFlashTime);
                            crossPhasePO.setMinGreenTime(minGreenTime);
                            crossPhasePO.setMaxGreenTime(maxGreenTime);
                            crossPhasePO.setPhaseTime(greenTime + greenFlashTime + yellowTime + redTime
                                    + yellowFlashTime + redFlashTime);
                            // 插入相位表记录，获取ID
                            crossPhaseMapper.insertOne(crossPhasePO);
                            Integer phaseId = crossPhasePO.getId();
                            if (phaseType == 1) { // 实相位
                                // 更新相位灯组关系表
                                updateCrossPhaseLights(crossId, phase, phaseId);
                            }
                        } else { // 伴随相位
                            CrossPhasePO crossPhasePO = new CrossPhasePO();
                            crossPhasePO.setPhaseNo(phaseNo);
                            crossPhasePO.setPhaseType(phaseType);
                            crossPhasePO.setSort(j + 1);
                            crossPhasePO.setCrossId(crossId);
                            crossPhasePO.setPlanId(schemeId);
                            crossPhasePO.setRingNo(ringNo);
                            crossPhasePO.setGroupNo(groupNo);
                            // 控制模式
                            crossPhasePO.setControlMode(ControlModeConst.FIXED_CYCLE);
                            Integer coordGreenTime = 0;
                            Integer coordGreenFlashTime = 0;
                            Integer coordYelloFlashTime = 0;
                            Integer coordRedFlashTime = 0;
                            Integer coordYellowTime = 0;
                            Integer coordRedTime = 0;
                            Integer coordMinGreenTime = 0;
                            Integer coordMaxGreenTime = 0;
                            List<String> superPhaseNoList = phase.getAccompanyPhaseNoList();
                            if (superPhaseNoList != null) {
                                for (String superPhaseNo : superPhaseNoList) {
                                    SaveSchemeConfigDTO.PhaseListElement phaseListElement = phaseNoMap.get(superPhaseNo);
                                    // 时长加总
                                    coordGreenTime += phaseListElement.getGreenTime();
                                    coordGreenFlashTime += phaseListElement.getGreenFlashTime();
                                    coordYelloFlashTime += phaseListElement.getYellowFlashTime();
                                    coordRedFlashTime += phaseListElement.getRedFlashTime();
                                    coordYellowTime += phaseListElement.getYellowTime();
                                    coordRedTime += phaseListElement.getRedTime();
                                    coordMinGreenTime += phaseListElement.getMinGreenTime();
                                    coordMaxGreenTime += phaseListElement.getMaxGreenTime();
                                }
                                crossPhasePO.setGreenTime(coordGreenTime);
                                crossPhasePO.setGreenFlashTime(coordGreenFlashTime);
                                crossPhasePO.setYellowFlashTime(coordYelloFlashTime);
                                crossPhasePO.setRedFlashTime(coordRedFlashTime);
                                crossPhasePO.setYellowTime(coordYellowTime);
                                crossPhasePO.setRedTime(coordRedTime);
                                crossPhasePO.setMinGreenTime(coordMinGreenTime);
                                crossPhasePO.setMaxGreenTime(coordMaxGreenTime);
                                crossPhasePO.setPhaseTime(coordGreenTime + coordGreenFlashTime + coordYelloFlashTime
                                        + coordRedFlashTime + coordYellowTime + coordRedTime);
                                // 插入相位表记录获取ID
                                crossPhaseMapper.insertOne(crossPhasePO);
                                Integer accompanyPhaseId = crossPhasePO.getId();

                                // 更新伴随相位母相位关系表
                                List<CrossAccompanyPhasePO> crossAccompanyPhasePOList = new ArrayList<>();
                                for (String superPhaseNo : superPhaseNoList) {
                                    CrossAccompanyPhasePO crossAccompanyPhasePO = new CrossAccompanyPhasePO();
                                    crossAccompanyPhasePO.setCrossId(crossId);
                                    crossAccompanyPhasePO.setPhaseId(accompanyPhaseId);
                                    Integer superPhaseId = crossPhaseMapper.selectIdByCrossIdAndPhaseNo(
                                            crossId, schemeId, superPhaseNo);
                                    crossAccompanyPhasePO.setSuperId(superPhaseId);
                                    crossAccompanyPhasePOList.add(crossAccompanyPhasePO);
                                }
                                crossAccompanyPhaseMapper.insertBatch(crossAccompanyPhasePOList);
                                // 更新相位灯组关系表
                                updateCrossPhaseLights(crossId, phase, accompanyPhaseId);
                            }
                        }
                    }
                }
            }
            // 处理特殊控制
            List<SaveSchemeConfigDTO.PhaseListElement> specialControlModeList = phaseScheme.getSpecialControlModeList();
            if (specialControlModeList != null) {
                for (int i = 0; i < specialControlModeList.size(); i++) {
                    SaveSchemeConfigDTO.PhaseListElement phase = specialControlModeList.get(i);
                    String phaseNo = phase.getPhaseNo();
                    try {
                        Integer.parseInt(phaseNo);
                    } catch (Exception e) {
                        throw new RuntimeException("相位编号必须为数字");
                    }
                    testPhaseNo(phaseNoSet, phaseNo);
                    CrossPhasePO crossPhasePO = new CrossPhasePO();
                    crossPhasePO.setPhaseNo(phaseNo);
                    crossPhasePO.setSort(i + 1);
                    crossPhasePO.setCrossId(crossId);
                    crossPhasePO.setPlanId(schemeId);
                    crossPhasePO.setPhaseType(1);
                    // 控制模式
                    if (phase.getYellowFlash() == 1) {
                        crossPhasePO.setControlMode(7);
                    } else if (phase.getIsRed() == 1) {
                        crossPhasePO.setControlMode(4);
                    } else if (phase.getIsGreen() == 1) {
                        crossPhasePO.setControlMode(2);
                    } else if (phase.getGreenFlash() == 1) {
                        crossPhasePO.setControlMode(6);
                    }
                    // 插入相位表记录，获取ID
                    crossPhaseMapper.insertOne(crossPhasePO);
                    Integer phaseId = crossPhasePO.getId();
                    // 更新相位灯组关系表
                    updateCrossPhaseLights(crossId, phase, phaseId);
                }
            }
        }
    }

    private static void testPhaseNo(Set<String> phaseNoSet, String phaseNo) throws PhaseNoNotUniqueException {
        boolean isUnique = phaseNoSet.add(phaseNo);
        if (!isUnique) {
            throw new PhaseNoNotUniqueException("同一方案中相位号不可重复");
        }
    }

    private void updateCrossPhaseLights(String crossId,
                                        SaveSchemeConfigDTO.PhaseListElement phase,
                                        Integer phaseId) {
        Set<CrossPhaseLightsPO> crossPhaseLightsPOSet = new HashSet<>();
        List<SaveSchemeConfigDTO.DirListElement> dirList = phase.getDirList();
        for (SaveSchemeConfigDTO.DirListElement dirListElement : dirList) {
            Integer dir = dirListElement.getDir();
            // 更新行人相位-灯组关系
            Integer hasPersonPhase = dirListElement.getHasPersonPhase();
            if (hasPersonPhase == 1) {
                List<BaseCrossLightsPO> crossLightsPOList = baseCrossLightsMapper.selectByCrossIdAndDir(crossId, dir);
                for (BaseCrossLightsPO crossLightsPO : crossLightsPOList) {
                    Integer type = crossLightsPO.getType();
                    if (type >= 20 && type < 30) { // 行人灯
                        Integer lightId = crossLightsPO.getId();
                        CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
                        crossPhaseLightsPO.setCrossId(crossId);
                        crossPhaseLightsPO.setLightsId(lightId);
                        crossPhaseLightsPO.setPhaseId(phaseId);
                        crossPhaseLightsPOSet.add(crossPhaseLightsPO);
                    }
                }
            }
            List<SaveSchemeConfigDTO.LaneListElement> laneList = dirListElement.getLaneList();
            for (SaveSchemeConfigDTO.LaneListElement laneListElement : laneList) {
                CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
                Integer category = laneListElement.getLaneType();
                Integer turn = laneListElement.getDirection();
                // 根据车道获得灯组ID
                String laneId = laneInfoMapper.selectIdByCrossIdDirTurn(crossId, dir, turn, category);
                Integer lightsId = baseCrossLaneLightsMapper.selectLightsIdByLaneId(crossId, laneId);
                if (lightsId != null) {
                    crossPhaseLightsPO.setLightsId(lightsId);
                }
                crossPhaseLightsPO.setPhaseId(phaseId);
                crossPhaseLightsPO.setCrossId(crossId);
                crossPhaseLightsPOSet.add(crossPhaseLightsPO);
            }
        }
        baseCrossPhaseLightsMapper.insertBatch(crossPhaseLightsPOSet);
    }

    private void updateCrossScheme(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        String crossId = saveSchemeConfigDTO.getCrossId();
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = saveSchemeConfigDTO.getPhaseSchemeList();
        // 删除数据库里有，但是DTO里没有的方案ID对应的记录
        Set<Integer> existedIds = baseCrossSchemeMapper.selectIdsByCrossId(crossId);
        if (CollectionUtil.isNotEmpty(existedIds)) {
            Set<Integer> dtoIds = phaseSchemeList.stream()
                    .map(SaveSchemeConfigDTO.PhaseScheme::getSchemeId)
                    .collect(Collectors.toSet());
            if (CollectionUtil.isNotEmpty(dtoIds)) {
                existedIds.removeAll(dtoIds);
                if (CollectionUtil.isNotEmpty(existedIds)) {
                    baseCrossSchemeMapper.deleteByIds(existedIds);
                }
            }
        }
        for (SaveSchemeConfigDTO.PhaseScheme phaseScheme : phaseSchemeList) {
            // 构造PO
            BaseCrossSchemePO baseCrossSchemePO = new BaseCrossSchemePO();
            String schemeNo = phaseScheme.getSchemeNo();// 方案号
            baseCrossSchemePO.setSchemeNo(schemeNo);
            String name = phaseScheme.getName(); // 方案名称
            baseCrossSchemePO.setName(name);
            baseCrossSchemePO.setCrossId(crossId);
            baseCrossSchemePO.setCycle(phaseScheme.getCycle());
            baseCrossSchemePO.setOffset(phaseScheme.getOffset());
            Integer offset = phaseScheme.getOffset();
            if (offset != null) {
                baseCrossSchemePO.setOffset(offset);
            }
            baseCrossSchemePO.setSource(2); // 来自平台
            baseCrossSchemePO.setIsDeleted(0);
            // 新增或修改数据
            Integer schemeId = phaseScheme.getSchemeId();
            if (ObjectUtil.isEmpty(schemeId)) {
                // 没有方案ID视为新增
                baseCrossSchemeMapper.insertOne(baseCrossSchemePO);
            } else {
                // 有方案ID视为修改
                baseCrossSchemePO.setId(schemeId);
                baseCrossSchemeMapper.update(baseCrossSchemePO);
            }
        }
    }
}
